/*  Copyright 1987 Bart Miller.  All rights reserved.
 *  Explicit permission required to redistributed this software.
 */
/****************************************************************
 *
 * This module contains the routines necessary to locate a 
 * running swb server.
 *
 *	ConnectToServ - return a socket to a running swb server
 *	ReadHosts -	read in posible swb hosts, convert to
 *			inet addresses and store in a table 
 *			(table defined in "locate.h")
 *
 ****************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>

#include "swbmsg.h"
#include "table.h"
#include "locate.h"


/*
 *		*------------------------------*
 *		|                              |
 *		|      ConnectToServ           |
 *		|                              |
 *		*------------------------------*
 */

long
ConnectToServ ()
{
    struct sockaddr_in sa;
    struct servent *sp;
    int servsock;
    int i;

    struct servent *getservbyname();

    /* get port # of server */
    if ((sp = getservbyname (SERVICENAME, "tcp")) == 0) {
	perror ("can't find swb");
	exit (-1);
    }

    /* if address to server exists, then use it to attempt connection */
    if (boundaddr > 0) {
    	if ((servsock = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
		perror ("can't create socket to swb");
		exit (-1);
    	}

	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = boundaddr;
#ifdef TESTMODE
	sa.sin_port = SERV_PORT;
#else
	sa.sin_port = sp->s_port;
#endif

	if (connect (servsock, &sa, sizeof(sa)) >= 0)
		return (servsock);  /* connection succeded */
	else 
		close (servsock);
    }
    
    /* have to broadcast to find a server */

    ReadHosts ();  /* read in file of possible hosts */

    sa.sin_family = AF_INET;
#ifdef TESTMODE
    sa.sin_port = SERV_PORT;
#else
    sa.sin_port = sp->s_port;
#endif

    for ( i=0; i < numhosts; i++) {

	sa.sin_addr.s_addr = hostaddrs[i];

    	if ((servsock = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
		perror ("can't create socket to swb");
		exit (-1);
    	}

	if (connect (servsock, &sa, sizeof(sa)) >= 0) {
		boundaddr = sa.sin_addr.s_addr;
		return (servsock);
	}
	else	
		close (servsock);
    }
    
    /* no server is alive */
    return (-1);

}

/*
 *		*------------------------------*
 *		|                              |
 *		|      ReadHosts               |
 *		|                              |
 *		*------------------------------*
 */

ReadHosts()
{
    int i;
    FILE *fp;
    servname host;
    struct hostent *he;

    FILE *fopen();
    struct hostent *gethostbyname();

    if ((fp = fopen (SWBHOSTFILE, "r")) == NULL) {
	perror (SWBHOSTFILE);
	exit (-1);
    }

    for (i = 0; fscanf (fp, "%s", host) != EOF; i++) {
	he = gethostbyname (host);
	hostaddrs[i] = ((struct in_addr *)(he->h_addr)) -> s_addr;
    }

    fclose (fp);	
    numhosts = i;
}
