/*  Copyright 1987 Bart Miller.  All rights reserved.
 *  Explicit permission required to redistributed this software.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <signal.h>
#include "swbmsg.h"
#include "table.h"

#define NUMTRIES 4

#define	TRUE	1
#define	OK	1
#define GENESIS	2

/*int	(*signal())();*/
char	*timestamp();



main (argc, argv)
    int argc;
    char *argv[];
{
	long servsock;
	struct sigvec vec;
	void (*PipeHandler)();

	if (fork () != 0)
	    exit (0);

	/*
	 *	set up to catch ^C
	 */
	/*
	vec.sv_mask = 0;
	vec.sv_onstack = 0;
	vec.sv_handler = KillHandler;
	if (sigvec(SIGQUIT, &vec, 0) < 0)
	{
		perror ("in sigvec");
		exit (0);
	}
	*/

	/*
	 *	set up to catch broken pipes
	 */
	vec.sv_mask = 0;
	vec.sv_onstack = 0;
	vec.sv_handler = PipeHandler;
	if (sigvec(SIGPIPE, &vec, 0) < 0)
	{
		perror ("in sigvec");
		exit (0);
	}

	
	/*
	fclose (stdin);
	fclose (stdout);
	*/

	freopen (SWBLOG, "a+", stderr);
	setbuf (stderr, NULL);

	/*
	 *	initialize the table of pending requests
	 */
	InitTable ();

	/*
	 *	initialize the list of servers
	 */
	InitServList ();

	fprintf (stderr, "%s: %s: Starting.\n", argv[0], timestamp());
	
	if (Initialize() == GENESIS)
		fprintf (stderr, "%s: Genesis mode\n", argv[0]);

	servsock = CreateDaemonSock (argc, argv);
	fprintf (stderr, "%s: %s: serv sock create (%d)\n", argv[0], 
		timestamp (), servsock);


	for (;;) {
	    WaitForRequest (servsock);
	}
}


/*
 *		*------------------------------*
 *		|                              |
 *		|        Initialize            |
 *		|                              |
 *		*------------------------------*
 */

int
Initialize()
{
	int i, rv;
	long servsock, mysock;
	u_short myport;
	struct sockaddr_in sa;
	struct servent *sp;
	struct inforesponse response;

	long ConnectToServ();
	long CreateBoundSocket ();

	
	while (TRUE) {
		if ((servsock = ConnectToServ()) < 0) 
			return (GENESIS);  /* no connection - */
					   /* I'm the only server */

		/* add the found server to my list of active ones */
		AddServer (servsock);
		SetMaskOn (servsock);
	
		/* now request list of active servers */
		if (ServSend(SERV_SEND_INFO, servsock, 0, 0, 0, 0) < 0) {
			close (servsock); 
			continue;  /* it died, try another */
		}

		if ((rv = read(servsock, &response, sizeof(response))) >= 0)
			break;
		else
			close (servsock);

	} /* while */
	
	/* got a valid response from the server */

	/* get port # of swb servers */
	if ((sp = getservbyname(SERVICENAME, "tcp")) == 0) {
		perror ("can't find swb");
		exit (-1);
	}
	sa.sin_family = AF_INET;
#ifdef TESTMODE
	sa.sin_port = SERV_PORT;
#else 
	sa.sin_port = sp->s_port;
#endif

	/* now form a connection to each active server */
	for (i = 0; i < response.numaddrs; i++) {
		if ((mysock = CreateBoundSocket(&myport)) < 0) {
			fputs("can't create new port\n", stderr);
			exit (-1);
		}
		/* get next address from list */
		sa.sin_addr.s_addr = response.servaddr[i];

		/* try to connect */
		if (connect(mysock, &sa, sizeof(sa)) >= 0) {
			if (ServSend(SERV_NEWSERV, mysock, 0, 0, 0, 0) >= 0) {
				AddServer (mysock);
				SetMaskOn (mysock);
			}
			else
				close (mysock);
		}
		else
			close (mysock);
	} 

	return (OK);
}


int
CreateDaemonSock (argc, argv)
    int argc;
    char *argv[];
{
	struct sockaddr_in sa;
	struct servent *sp;
	int servsock;
	long tries=0;

	struct servent *getservbyname();

	if ((sp = getservbyname (SERVICENAME, "tcp")) == 0) {
		perror ("can't find SWB service");
		exit (-1);
	}

	if ((servsock = socket (AF_INET, SOCK_STREAM, 0)) == -1) {
	perror ("can't create daemon socket");
	exit (-1);
	}

	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = INADDR_ANY;
#ifdef TESTMODE
	sa.sin_port = SERV_PORT;
	fprintf(stderr, "server port number = %d\n", sa.sin_port);
#else
	sa.sin_port = sp->s_port;
#endif
dumbness:
	if (bind (servsock, &sa, sizeof (sa)))
	{
		if (++tries < NUMTRIES)
		{
			sleep (15);	/* wait a bit */
			fprintf (stderr, "%s: %s: bind problem: trying again\n",
				argv [0], timestamp ());
			goto dumbness;
		}
		perror ("binding sock name");
		exit (-1);
	}
	listen (servsock, 5);

	return (servsock);
}


long
CreateBoundSocket (myport) 
    u_short *myport;
{
    struct sockaddr_in sa;
    long sock, len;

    sock = socket (AF_INET, SOCK_STREAM, 0);

    sa.sin_family = AF_INET;
    sa.sin_addr.s_addr = INADDR_ANY;
    sa.sin_port = 0;

    if (bind (sock, &sa, sizeof (sa)))
    {
	perror ("binding");
	return (-1);
    }

    len = sizeof (sa);
    if (getsockname (sock, &sa, &len)) {
	perror ("getting sock name");
	return (-1);
    }

    *myport = sa.sin_port;
    return (sock);
}



char *
timestamp ()
{
    long daytime;
    char *dp;

    long time ();
    char *ctime ();

    daytime = time (0);
    dp = ctime (&daytime);
    dp [strlen (dp) - 1] = '\000'; 
    return (dp);
}
