/*  Copyright 1987 Bart Miller.  All rights reserved.
 *  Explicit permission required to redistributed this software.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netdb.h>
#include <stdio.h>
#include <netinet/in.h>

#include "swbmsg.h"
#include "table.h"

struct serventry *servlist;

/*
 *		-------------------------
 *		|			|
 *		|	InitServList	|
 *		|			|
 *		-------------------------
 */

InitServList ()
{
#ifdef DEBUG
	printf ("Entering InitServList\n");
#endif
	servlist = (struct serventry *) 0;
}


/*
 *		-------------------------
 *		|			|
 *		|	GetFirstServer	|
 *		|			|
 *		-------------------------
 */

struct serventry *
GetFirstServer()
{
	return (servlist);
}



/*
 *		-------------------------
 *		|			|
 *		|	AddServer	|
 *		|			|
 *		-------------------------
 */

AddServer (sock)
    long sock;
{
	struct serventry *se;

#ifdef DEBUG
	printf ("Entering AddServer\n");
#endif
	se = (struct serventry *) malloc (sizeof (struct serventry));
	se->sock = sock;
	se->next = servlist;
	servlist = se;
#ifdef DEBUG
	printf ("Leaving AddServer\n");
#endif
}


/*
 *		-------------------------
 *		|			|
 *		|	FindServer	|
 *		|			|
 *		-------------------------
 */

struct serventry *
FindServer (sock)
    long sock;
{
	struct serventry *se;

	se = servlist;
	while (se != (struct serventry *) 0)
	{
		if (se->sock == sock)
			return (se);
		else
			se = se->next;
	}
	return ((struct serventry *) 0);
}


/*
 *		-------------------------
 *		|			|
 *		|	DeleteServer	|
 *		|			|
 *		-------------------------
 */

DeleteServer(entry)
   struct serventry *entry;
{
	struct serventry *se;
	struct serventry *prev_se;

	DelServEntry(entry->sock);
	if (servlist == entry)
	{
		servlist = servlist->next;
		free(entry);
		return;
	} else {
		se = servlist;
		while (se != (struct serventry *) 0)
		{
			if (se == entry)
			{
				prev_se->next = se->next;
				free(se);
				return;
			}
			prev_se = se;
			se = se->next;
		}
	}
	fprintf (stderr, "help! can't find server to delete\n");
}

/*
 *		-------------------------
 *		|			|
 *		|	ServSend	|
 *		|			|
 *		-------------------------
 */

ServSend (func, servsock, me, her, myport, myhost)
    long func;
    long servsock;
    servname me;
    servname her;
    u_short myport;
    struct in_addr myhost;
{
	struct servrequest request;
        struct serventry *FindServer();

	request.function = func;
	strcpy (&request.myname[0], &me[0]);
	request.myport = myport;
	strcpy (&request.hername[0], &her[0]);
	request.hostid = 0;
	request.myaddr = myhost;

	if (send (servsock, &request, sizeof (struct servrequest), 0) < 0)
	{
		perror ("Error in Sending to Server");
		DeleteServer (FindServer(servsock));
		return -1;
	}
	return 0;
}




/*
 *		-------------------------
 *		|			|
 *		|	Broadcast	|
 *		|			|
 *		-------------------------
 */

Broadcast (request)
    struct servrequest *request;
{
	struct serventry *server;

	server = servlist;
	while (server != (struct serventry *) 0)
	{
		if (send (server->sock, request, sizeof(struct servrequest), 0) < 0)
		{
			perror ("Error in Broadcast to Servers");
			DeleteServer(server);
			return -1;
		}
		server = server->next;
	}
	return 0;
}



/*
 *		-------------------------
 *		|			|
 *		|	PropagateUpdate	|
 *		|			|
 *		-------------------------
 */

PropagateUpdate (request, sock)
    struct servrequest *request;
    long sock;
{

	struct in_addr ha;
	/*
	 *	package meet request and broadcast it to all other servers
	 */
	SockToHostAddr (sock, &ha);
	request->myaddr = ha;
	request->hostid = gethostid();
	if (Broadcast (request) < 0)
	{
		perror ("on Broadcast");
		return -1;
	}
	return 0;
}



/*
 *		-------------------------
 *		|			|
 *		|	CleanUp 	|
 *		|			|
 *		-------------------------
 */

CleanUp (me, her)
    servname me, her;
{
	struct servrequest request;
	/*
	 *	package DeleteEntry request and broadcast it
	 */

	request.function = SERV_CLEANUP;
	strcpy (request.myname, me);
	request.myport = 0;
	strcpy (request.hername, her);
	request.hostid = 0;
	/*
	request.myaddr = 0;
	*/
	if (Broadcast (&request) < 0)
	{
		perror ("on Broadcast");
		return -1;
	}
	return 0;
}


KillHandler (sig, code, scp)
    int sig, code;
    struct sigcontext *scp;
{
	fprintf (stderr, "kill server\n");
	exit (3);	/* close all open file descriptors */
}



PipeHandler (sig, code, scp)
    int sig, code;
    struct sigcontext *scp;
{
	fprintf (stderr, "broken pipe\n");
}
